from __future__ import annotations

import json
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from . import _pydmg

PathLike = Union[str, Path]


def _as_path(path: PathLike) -> str:
    return str(Path(path))


def inspect(path: PathLike) -> Dict[str, Any]:
    """Return a full DMG inspection payload.

    The payload includes trailer details, plist metadata, checksum validation,
    partition tables/chunks, and extracted metadata candidates.
    """
    return json.loads(_pydmg.inspect_json(_as_path(path)))


def inspect_filesystems(path: PathLike) -> Dict[str, Any]:
    """Return filesystem metadata discovered in DMG partitions.

    Includes FAT metadata from partition payloads and HFS+/APFS metadata
    via auto-detected Apple filesystem parsing.
    """
    return json.loads(_pydmg.inspect_filesystems_json(_as_path(path)))


def list_partitions(path: PathLike) -> List[Dict[str, Any]]:
    """Return partition metadata and chunk tables for a DMG."""
    return json.loads(_pydmg.list_partitions_json(_as_path(path)))


def inspect_gpt(
    path: PathLike,
    partition_index: Optional[int] = None,
    strict: bool = False,
) -> Dict[str, Any]:
    """Inspect GPT metadata from DMG partition payloads.

    If ``partition_index`` is ``None``, all partitions are scanned and the
    first GPT table found is returned. If none are found, ``has_gpt`` is false.
    Set ``strict=True`` to raise an exception when GPT data is not found.
    """
    return json.loads(_pydmg.inspect_gpt_json(_as_path(path), partition_index, strict))


def read_partition(path: PathLike, index: int) -> bytes:
    """Read and decompress a partition payload by index."""
    return _pydmg.read_partition(_as_path(path), index)


def extract_partition(path: PathLike, index: int, output_path: PathLike) -> int:
    """Write a partition payload to disk and return number of bytes written."""
    return int(_pydmg.extract_partition(_as_path(path), index, _as_path(output_path)))


def compute_data_checksum(path: PathLike) -> int:
    """Compute CRC32 for the DMG data fork."""
    return int(_pydmg.compute_data_checksum(_as_path(path)))


def verify_data_checksum(path: PathLike) -> bool:
    """Compare computed and trailer-declared data fork checksums."""
    return bool(_pydmg.verify_data_checksum(_as_path(path)))


def list_fat32_entries(path: PathLike, partition_index: int = 1) -> List[Dict[str, Any]]:
    """List files/directories inside a FAT32 partition."""
    return json.loads(_pydmg.list_fat32_entries_json(_as_path(path), partition_index))


def extract_fat32(
    path: PathLike,
    output_dir: PathLike,
    partition_index: int = 1,
    overwrite: bool = False,
) -> List[str]:
    """Extract files from a FAT32 partition into ``output_dir``."""
    return json.loads(
        _pydmg.extract_fat32_json(
            _as_path(path),
            _as_path(output_dir),
            partition_index,
            overwrite,
        )
    )


def list_apple_entries(path: PathLike, directory_path: str = "/") -> List[Dict[str, Any]]:
    """List entries from auto-detected HFS+/APFS filesystem."""
    return json.loads(_pydmg.list_apple_entries_json(_as_path(path), directory_path))


def read_apple_file(path: PathLike, file_path: str) -> bytes:
    """Read a file from auto-detected HFS+/APFS filesystem."""
    return _pydmg.read_apple_file(_as_path(path), file_path)


def extract_apple_file(path: PathLike, file_path: str, output_path: PathLike) -> int:
    """Extract a file from auto-detected HFS+/APFS filesystem to disk."""
    return int(_pydmg.extract_apple_file(_as_path(path), file_path, _as_path(output_path)))


def create_dmg(
    source_dir: PathLike,
    output_path: PathLike,
    volume_label: str = "PYDMG",
    total_sectors: int = 32768,
) -> None:
    """Create a DMG containing ``source_dir`` using apple-dmg's FAT32 writer.

    ``total_sectors`` defines total image size in 512-byte sectors.
    """
    _pydmg.create_dmg(_as_path(source_dir), _as_path(output_path), volume_label, total_sectors)


class DmgImage:
    """Convenience object around a DMG file path."""

    def __init__(self, path: PathLike):
        self.path = Path(path)

    def inspect(self) -> Dict[str, Any]:
        return inspect(self.path)

    def inspect_filesystems(self) -> Dict[str, Any]:
        return inspect_filesystems(self.path)

    def partitions(self) -> List[Dict[str, Any]]:
        return list_partitions(self.path)

    def inspect_gpt(
        self,
        partition_index: Optional[int] = None,
        strict: bool = False,
    ) -> Dict[str, Any]:
        return inspect_gpt(self.path, partition_index=partition_index, strict=strict)

    def metadata(self) -> Dict[str, Any]:
        return self.inspect().get("metadata_candidates", {})

    def read_partition(self, index: int) -> bytes:
        return read_partition(self.path, index)

    def extract_partition(self, index: int, output_path: PathLike) -> int:
        return extract_partition(self.path, index, output_path)

    def checksum_valid(self) -> bool:
        return verify_data_checksum(self.path)

    def list_fat32_entries(self, partition_index: int = 1) -> List[Dict[str, Any]]:
        return list_fat32_entries(self.path, partition_index=partition_index)

    def extract_fat32(
        self,
        output_dir: PathLike,
        partition_index: int = 1,
        overwrite: bool = False,
    ) -> List[str]:
        return extract_fat32(
            self.path,
            output_dir=output_dir,
            partition_index=partition_index,
            overwrite=overwrite,
        )

    def list_apple_entries(self, directory_path: str = "/") -> List[Dict[str, Any]]:
        return list_apple_entries(self.path, directory_path=directory_path)

    def read_apple_file(self, file_path: str) -> bytes:
        return read_apple_file(self.path, file_path=file_path)

    def extract_apple_file(self, file_path: str, output_path: PathLike) -> int:
        return extract_apple_file(self.path, file_path=file_path, output_path=output_path)



#!/usr/bin/env python3
"""Validate Rust dependency licenses against a permissive allowlist."""

from __future__ import annotations

import json
import re
import subprocess
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional, Tuple


ALLOWED_LICENSE_IDS = {
    "MIT",
    "Apache-2.0",
    "BSD-2-Clause",
    "BSD-3-Clause",
    "0BSD",
    "ISC",
    "Unlicense",
    "Zlib",
    "Unicode-3.0",
}

ALLOWED_EXCEPTIONS = {
    "LLVM-exception",
}

EXPRESSION_NORMALIZATION = {
    "MIT/Apache-2.0": "MIT OR Apache-2.0",
    "Apache-2.0/MIT": "Apache-2.0 OR MIT",
    "MIT/Apache-2.0 OR Apache-2.0/MIT": "MIT OR Apache-2.0",
}

TOKEN_RE = re.compile(r"\(|\)|AND|OR|WITH|[A-Za-z0-9.+-]+")


@dataclass
class LicenseNode:
    kind: str
    value: Optional[str] = None
    left: Optional["LicenseNode"] = None
    right: Optional["LicenseNode"] = None


def normalize_expression(expr: str) -> str:
    normalized = expr.strip()
    for original, replacement in EXPRESSION_NORMALIZATION.items():
        normalized = normalized.replace(original, replacement)
    return normalized


def tokenize(expr: str) -> List[str]:
    return TOKEN_RE.findall(expr)


class Parser:
    def __init__(self, tokens: List[str]) -> None:
        self.tokens = tokens
        self.index = 0

    def parse(self) -> LicenseNode:
        node = self.parse_or()
        if self.index != len(self.tokens):
            raise ValueError(f"unexpected token: {self.tokens[self.index]}")
        return node

    def parse_or(self) -> LicenseNode:
        node = self.parse_and()
        while self.peek() == "OR":
            self.index += 1
            node = LicenseNode("OR", left=node, right=self.parse_and())
        return node

    def parse_and(self) -> LicenseNode:
        node = self.parse_with()
        while self.peek() == "AND":
            self.index += 1
            node = LicenseNode("AND", left=node, right=self.parse_with())
        return node

    def parse_with(self) -> LicenseNode:
        node = self.parse_primary()
        if self.peek() == "WITH":
            self.index += 1
            exception = self.consume_identifier("exception")
            return LicenseNode("WITH", value=exception, left=node)
        return node

    def parse_primary(self) -> LicenseNode:
        token = self.peek()
        if token is None:
            raise ValueError("unexpected end of expression")
        if token == "(":
            self.index += 1
            node = self.parse_or()
            if self.peek() != ")":
                raise ValueError("missing closing ')'")
            self.index += 1
            return node
        if token in {"AND", "OR", "WITH", ")"}:
            raise ValueError(f"unexpected token: {token}")
        self.index += 1
        return LicenseNode("ID", value=token)

    def consume_identifier(self, label: str) -> str:
        token = self.peek()
        if token is None or token in {"AND", "OR", "WITH", "(", ")"}:
            raise ValueError(f"expected {label}")
        self.index += 1
        return token

    def peek(self) -> Optional[str]:
        if self.index >= len(self.tokens):
            return None
        return self.tokens[self.index]


def expression_is_allowed(node: LicenseNode) -> bool:
    if node.kind == "ID":
        return bool(node.value in ALLOWED_LICENSE_IDS)
    if node.kind == "WITH":
        if not node.left or not expression_is_allowed(node.left):
            return False
        return bool(node.value in ALLOWED_EXCEPTIONS)
    if node.kind == "AND":
        return bool(node.left and node.right and expression_is_allowed(node.left)
                    and expression_is_allowed(node.right))
    if node.kind == "OR":
        return bool(node.left and node.right and
                    (expression_is_allowed(node.left) or expression_is_allowed(node.right)))
    return False


def load_metadata(root: Path) -> dict:
    command = ["cargo", "metadata", "--format-version=1"]
    if (root / "Cargo.lock").exists():
        command.append("--locked")

    result = subprocess.run(
        command,
        cwd=root,
        check=False,
        capture_output=True,
        text=True,
    )
    if result.returncode != 0:
        stderr = result.stderr.strip() or "unknown error"
        raise RuntimeError(f"cargo metadata failed: {stderr}")
    return json.loads(result.stdout)


def evaluate_license(expr: str) -> Tuple[bool, str]:
    normalized = normalize_expression(expr)
    tokens = tokenize(normalized)
    if not tokens:
        return False, "empty license expression"
    try:
        tree = Parser(tokens).parse()
    except ValueError as exc:
        return False, f"unable to parse SPDX expression '{normalized}': {exc}"
    if expression_is_allowed(tree):
        return True, ""
    return (
        False,
        "no allowed permissive license path in expression "
        f"'{normalized}' (allowed IDs: {sorted(ALLOWED_LICENSE_IDS)})",
    )


def main() -> int:
    root = Path(__file__).resolve().parents[1]
    metadata = load_metadata(root)
    failures: List[str] = []
    checked = 0

    for pkg in metadata.get("packages", []):
        if pkg.get("source") is None:
            continue
        checked += 1
        name = pkg.get("name", "<unknown>")
        version = pkg.get("version", "<unknown>")
        expr = pkg.get("license")
        if not expr:
            failures.append(f"{name} {version}: missing license expression")
            continue
        ok, reason = evaluate_license(expr)
        if not ok:
            failures.append(f"{name} {version}: {reason}")

    if failures:
        print("license policy check failed for Rust dependencies:", file=sys.stderr)
        for item in failures:
            print(f"- {item}", file=sys.stderr)
        return 1

    print(f"license policy check OK ({checked} third-party Rust crates checked)")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
"""Build pydoc HTML docs for pydmg."""

from __future__ import annotations

import argparse
import os
import pydoc
from pathlib import Path


def main() -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--output-dir",
        default=".",
        help="Directory where pydoc HTML will be written (default: current directory)",
    )
    parser.add_argument(
        "--cleanup",
        action="store_true",
        help="Delete the generated HTML after verifying it was created",
    )
    args = parser.parse_args()

    output_dir = Path(args.output_dir).resolve()
    output_dir.mkdir(parents=True, exist_ok=True)

    output_filename = "pydmg.html"
    old_cwd = Path.cwd()
    try:
        os.chdir(output_dir)
        pydoc.writedoc("pydmg")
    finally:
        os.chdir(old_cwd)

    output_path = output_dir / output_filename
    if not output_path.exists():
        raise RuntimeError(f"pydoc output was not created: {output_path}")

    print(f"pydoc build OK: {output_path}")

    if args.cleanup:
        output_path.unlink()
        print(f"pydoc cleanup OK: {output_path}")

    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
"""Validate release versions across project metadata files."""

from __future__ import annotations

import argparse
import sys
import tomllib
from pathlib import Path


def read_pyproject_version(path: Path) -> str:
    with path.open("rb") as handle:
        data = tomllib.load(handle)
    try:
        return str(data["project"]["version"])
    except KeyError as exc:
        raise RuntimeError(f"missing project.version in {path}") from exc


def read_cargo_version(path: Path) -> str:
    with path.open("rb") as handle:
        data = tomllib.load(handle)
    try:
        return str(data["package"]["version"])
    except KeyError as exc:
        raise RuntimeError(f"missing package.version in {path}") from exc


def normalize(version: str) -> str:
    return version.strip().removeprefix("v")


def main() -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--expected",
        default="",
        help="Expected release version, with or without leading v",
    )
    args = parser.parse_args()

    root = Path(__file__).resolve().parents[1]
    pyproject = root / "pyproject.toml"
    cargo = root / "Cargo.toml"

    py_version = normalize(read_pyproject_version(pyproject))
    cargo_version = normalize(read_cargo_version(cargo))

    if py_version != cargo_version:
        print(
            f"version mismatch: pyproject={py_version} cargo={cargo_version}",
            file=sys.stderr,
        )
        return 1

    expected = normalize(args.expected)
    if expected and py_version != expected:
        print(
            f"tag/version mismatch: expected={expected} project={py_version}",
            file=sys.stderr,
        )
        return 1

    print(f"version check OK: {py_version}")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import lzma
from pathlib import Path
from typing import Optional

import pytest

import pydmg


def _build_source_tree(tmp_path: Path) -> Path:
    source = tmp_path / "payload"
    source.mkdir()
    (source / "hello.txt").write_text("hello from pydmg\n", encoding="utf-8")
    docs = source / "docs"
    docs.mkdir()
    (docs / "readme.md").write_text("sample file\n", encoding="utf-8")
    return source


def _extract_fixture_xz(tmp_path: Path, fixture_name: str) -> Path:
    fixture_path = Path(__file__).parent / "fixtures" / fixture_name
    output = tmp_path / fixture_name.removesuffix(".xz")
    output.write_bytes(lzma.decompress(fixture_path.read_bytes()))
    return output


def _build_dmg(tmp_path: Path) -> tuple[Path, Path]:
    source = _build_source_tree(tmp_path)
    dmg = tmp_path / "sample.dmg"
    pydmg.create_dmg(source, dmg, volume_label="TESTVOL", total_sectors=32768)
    assert dmg.exists()
    assert dmg.stat().st_size > 0
    return source, dmg


def _find_first_regular_file(image_path: Path) -> Optional[str]:
    queue = ["/"]
    visited = set(queue)
    scanned_dirs = 0
    max_dirs = 512

    while queue and scanned_dirs < max_dirs:
        directory = queue.pop(0)
        scanned_dirs += 1
        entries = pydmg.list_apple_entries(image_path, directory_path=directory)

        for entry in entries:
            path = entry["path"]
            if entry["is_dir"]:
                if path not in visited:
                    visited.add(path)
                    queue.append(path)
            else:
                return path

    return None


def test_inspect_includes_core_sections(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    info = pydmg.inspect(dmg)

    assert info["koly"]["version"] == 4
    assert info["checksum"]["data_fork_checksum_matches"] is True
    assert isinstance(info["plist"], dict)
    assert isinstance(info["metadata_candidates"], dict)
    assert len(info["partitions"]) >= 2
    assert isinstance(info["fat_filesystems"], list)
    assert "apple_filesystem" in info
    assert isinstance(info["filesystem_errors"], list)


def test_inspect_gpt_non_strict_reports_absence(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    info = pydmg.inspect_gpt(dmg)

    assert info["has_gpt"] is False
    assert info["partition_index"] is None
    assert isinstance(info["errors"], list)
    assert info["errors"] == []


def test_inspect_gpt_strict_raises(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    with pytest.raises(RuntimeError):
        pydmg.inspect_gpt(dmg, strict=True)


def test_partition_read_and_extract(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    partition_bytes = pydmg.read_partition(dmg, 1)
    assert isinstance(partition_bytes, (bytes, bytearray))
    assert len(partition_bytes) > 0

    partition_file = tmp_path / "fat32_partition.bin"
    written = pydmg.extract_partition(dmg, 1, partition_file)
    assert written == len(partition_bytes)
    assert partition_file.read_bytes() == partition_bytes

    nested_partition_file = tmp_path / "nested" / "dir" / "partition.bin"
    nested_written = pydmg.extract_partition(dmg, 1, nested_partition_file)
    assert nested_written == len(partition_bytes)
    assert nested_partition_file.read_bytes() == partition_bytes


def test_fat32_listing_and_extraction(tmp_path: Path) -> None:
    source, dmg = _build_dmg(tmp_path)

    entries = pydmg.list_fat32_entries(dmg, partition_index=1)
    entry_paths = {item["path"] for item in entries}

    root_name = source.name
    assert f"{root_name}/hello.txt" in entry_paths
    assert f"{root_name}/docs/readme.md" in entry_paths

    out_dir = tmp_path / "extracted"
    extracted = pydmg.extract_fat32(dmg, out_dir, partition_index=1)
    assert f"{root_name}/hello.txt" in extracted

    assert (out_dir / root_name / "hello.txt").read_text(encoding="utf-8") == "hello from pydmg\n"
    assert (out_dir / root_name / "docs" / "readme.md").read_text(encoding="utf-8") == "sample file\n"

    with pytest.raises(RuntimeError):
        pydmg.extract_fat32(dmg, out_dir, partition_index=1, overwrite=False)

    extracted_overwrite = pydmg.extract_fat32(dmg, out_dir, partition_index=1, overwrite=True)
    assert f"{root_name}/hello.txt" in extracted_overwrite


def test_index_errors_are_exposed(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    with pytest.raises(IndexError):
        pydmg.read_partition(dmg, 99)

    with pytest.raises(IndexError):
        pydmg.list_fat32_entries(dmg, partition_index=99)

    with pytest.raises(IndexError):
        pydmg.inspect_gpt(dmg, partition_index=99)

    with pytest.raises(IndexError):
        pydmg.extract_fat32(dmg, tmp_path / "out", partition_index=99)


def test_checksum_helpers_match_inspect(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    checksum = pydmg.compute_data_checksum(dmg)
    valid = pydmg.verify_data_checksum(dmg)
    info = pydmg.inspect(dmg)

    assert checksum == info["checksum"]["computed_data_fork_crc32"]
    assert valid is True


def test_create_dmg_input_validation(tmp_path: Path) -> None:
    source = _build_source_tree(tmp_path)
    output = tmp_path / "invalid.dmg"

    with pytest.raises(ValueError):
        pydmg.create_dmg(source, output, total_sectors=0)

    with pytest.raises(ValueError):
        pydmg.create_dmg(tmp_path / "missing", output)

    file_source = tmp_path / "not_a_dir.txt"
    file_source.write_text("x", encoding="utf-8")
    with pytest.raises(ValueError):
        pydmg.create_dmg(file_source, output)


def test_inspect_filesystems_reports_fat_metadata(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    info = pydmg.inspect_filesystems(dmg)

    assert isinstance(info["fat_filesystems"], list)
    assert info["apple_filesystem"] is None
    assert isinstance(info["errors"], list)
    assert len(info["fat_filesystems"]) >= 1

    fat = info["fat_filesystems"][0]
    assert fat["fat_type"] in {"fat12", "fat16", "fat32"}
    assert fat["cluster_size"] is not None
    assert fat["cluster_size"] > 0
    assert fat["total_clusters"] is not None
    assert fat["total_clusters"] > 0

    inspect_info = pydmg.inspect(dmg)
    assert inspect_info["fat_filesystems"] == info["fat_filesystems"]


def test_fat32_fixture_detected_as_fat32() -> None:
    fixture = Path(__file__).parent / "fixtures" / "fat32-large-sample.dmg"
    info = pydmg.inspect_filesystems(fixture)
    fat_types = {entry["fat_type"] for entry in info["fat_filesystems"]}
    assert "fat32" in fat_types


def test_hfs_fixture_listing_and_extract(tmp_path: Path) -> None:
    image = _extract_fixture_xz(tmp_path, "hfsp-small.img.xz")

    info = pydmg.inspect_filesystems(image)
    apple = info["apple_filesystem"]
    assert apple is not None
    assert apple["fs_type"] == "hfsplus"

    root_entries = pydmg.list_apple_entries(image, "/")
    assert len(root_entries) > 0

    root_entries_via_empty = pydmg.list_apple_entries(image, "")
    assert len(root_entries_via_empty) == len(root_entries)

    file_path = _find_first_regular_file(image)
    assert file_path is not None

    payload = pydmg.read_apple_file(image, file_path)
    assert isinstance(payload, (bytes, bytearray))

    output_file = tmp_path / "hfs_extracted.bin"
    written = pydmg.extract_apple_file(image, file_path, output_file)
    assert written == len(payload)
    assert output_file.read_bytes() == payload

    with pytest.raises(RuntimeError):
        pydmg.read_apple_file(image, "/definitely/does/not/exist")


def test_apfs_fixture_listing_and_extract(tmp_path: Path) -> None:
    image = Path(__file__).parent / "fixtures" / "apfs-linearmouse-v0.10.2.dmg"

    info = pydmg.inspect_filesystems(image)
    apple = info["apple_filesystem"]
    assert apple is not None
    assert apple["fs_type"] == "apfs"

    entries = pydmg.list_apple_entries(image, "/")
    assert len(entries) > 0

    file_path = _find_first_regular_file(image)
    assert file_path is not None

    payload = pydmg.read_apple_file(image, file_path)
    out_file = tmp_path / "apfs_extracted.bin"
    written = pydmg.extract_apple_file(image, file_path, out_file)
    assert written == len(payload)
    assert out_file.read_bytes() == payload


def test_apple_filesystem_helpers_raise_when_absent(tmp_path: Path) -> None:
    _, dmg = _build_dmg(tmp_path)

    with pytest.raises(RuntimeError):
        pydmg.list_apple_entries(dmg)

    with pytest.raises(RuntimeError):
        pydmg.read_apple_file(dmg, "/hello.txt")

    with pytest.raises(RuntimeError):
        pydmg.extract_apple_file(dmg, "/hello.txt", tmp_path / "hello.txt")


def test_dmgimage_convenience_api(tmp_path: Path) -> None:
    source, dmg = _build_dmg(tmp_path)
    image = pydmg.DmgImage(dmg)

    assert image.checksum_valid() is True
    assert len(image.partitions()) >= 2
    assert f"{source.name}/hello.txt" in {
        entry["path"] for entry in image.list_fat32_entries(partition_index=1)
    }
    assert len(image.inspect_filesystems()["fat_filesystems"]) >= 1
    assert image.inspect_gpt()["has_gpt"] is False


def test_dmgimage_hfs_convenience_methods(tmp_path: Path) -> None:
    image_path = _extract_fixture_xz(tmp_path, "hfsp-small.img.xz")
    image = pydmg.DmgImage(image_path)

    fs_info = image.inspect_filesystems()
    assert fs_info["apple_filesystem"]["fs_type"] == "hfsplus"
    entries = image.list_apple_entries("/")
    assert len(entries) > 0

    file_path = _find_first_regular_file(image_path)
    assert file_path is not None
    blob = image.read_apple_file(file_path)
    out = tmp_path / "hfs_obj_extract.bin"
    written = image.extract_apple_file(file_path, out)
    assert written == len(blob)
    assert out.read_bytes() == blob



from __future__ import annotations

from importlib.metadata import PackageNotFoundError, version

from .core import (
    DmgImage,
    compute_data_checksum,
    create_dmg,
    extract_apple_file,
    extract_fat32,
    extract_partition,
    inspect_filesystems,
    inspect_gpt,
    inspect,
    list_apple_entries,
    list_fat32_entries,
    list_partitions,
    read_apple_file,
    read_partition,
    verify_data_checksum,
)

try:
    __version__ = version("pydmg")
except PackageNotFoundError:
    __version__ = "0.0.0"

__all__ = [
    "DmgImage",
    "inspect",
    "inspect_filesystems",
    "inspect_gpt",
    "list_partitions",
    "read_partition",
    "extract_partition",
    "list_fat32_entries",
    "extract_fat32",
    "list_apple_entries",
    "read_apple_file",
    "extract_apple_file",
    "create_dmg",
    "compute_data_checksum",
    "verify_data_checksum",
    "__version__",
]
